#' R script to accompany "Taurus of the tidepool? Inferring the function of cranial 
#' weapons in intertidal sculpins (Pisces: Cottoidea: Oligocottinae)"
#' 
#' Thaddaeus J. Buser, Victoria E. Kee, Rebecca C. Terry, Adam P. Summers, and Brian L. Sidlauskas
#' 
#' 
#' 
#' For questions, comments, or concerns regarding the following script, please
#' contact:
#' 
#' Thaddaeus Buser
#' Thaddaeus.Buser@gmail.com
#'
#'
#' The first thing you'll want to do is designate a working drive and
#' move all supplementary materials (1.1 - 1.9) into that drive. To
#' do so, create a new folder called "Cranial_Weapons_R" in a convenient
#' location such as the desktop of your computer. Next, move all nine
#' supplementary materials files into this folder. This may require you
#' to close this script and reopen it once you have moved it. Now, 
#' designate this new folder as your working drive by copying the file
#' pathway of the "Cranial_Weapons_R" folder into the "setwd()" function
#' below (e.g., setwd("//Users//thaddaeusbuser//Desktop//Cranial_Weapons_R"))
#' or, if you are working in R Studio, click on the "Session" menu, go down
#' to "Set Working Directory" and select "Choose Directory..." then navigate
#' to the "Cranial_Weapons_R" folder and select it. If you use the latter
#' option, simply delete the "setwd()" line below so that  you do not 
#' unintentionally call the function.
#
setwd()
#
#
library("geomorph")
library("phytools")
library("ape")
library("vegan")
#'
#' Now we'll read-in some custom functions that I developed. If you use them in your own work,
#' please cite geomorph, ape, vegan, as well as:
#' 
#' Buser, T.J., Burns, M.D. and López, J.A., 2017. Littorally adaptive? Testing the link 
#' between habitat, morphology, and reproduction in the intertidal sculpin subfamily 
#' Oligocottinae (Pisces: Cottoidea). PeerJ, 5, p.e3634.
#' 
#' and
#' 
#' Buser, T.J., Sidlauskas, B.L. and Summers, A.P., 2018. 2D or not 2D? Testing the 
#' utility of 2D vs. 3D landmark data in geometric morphometrics of the sculpin subfamily 
#' Oligocottinae (Pisces; Cottoidea). The Anatomical Record, 301(5), pp.806-818.
#' 
source("Supplementary_materials_1.2_Custom.functions.R") #Be sure keep the custom functions script in your working drive 
#
######### STEP 1: Geometric morphometric analyses ############
#' This section corresponds to the "Geometric morphometrics" section in the methods section.
#' 
#### STEP 1.1: Import landmark data and perform Procrustes analysis
#
# Let's read in the data for the left and right preopercles
OligoCombinedPreopercles <- readland.tps("Supplementary_materials_1.3_OligoLeftRightPOP.tps", specID = "ID")

# Read-in specimen data for each preopercle
spec.info <- read.csv("Supplementary_materials_1.4_specimen_info1.csv", header = T, row.names = 1) 
#
# Resample the semilandmark curve
#' Okay, looks good, let's resample the semilandmarks using my custom function
resample.test1 <- my.curve.resampler(landmark.array = OligoCombinedPreopercles, fixed.landmarks = 28, curve.resample = 60)
#
# Now let's try analyzing the semilandmarks as sliding along curves
curvesliders <- as.matrix(read.csv("my.slider.matrix.csv", header = T)) #Slider matrix is auot-generated by my curve resampler function
#
# Gonna double check the order
dimnames(resample.test1)[[3]] == row.names(spec.info) # No good
spec.info <- spec.info[dimnames(resample.test1)[[3]],] # Re-order info to match landmark data order
#
# Perform a generalized Procrustes analysis
oligocottinae.gpa <- gpagen(A=resample.test1, curves = curvesliders, ProcD = F) 
#
# Visualize shape variation using a principal component analysis, with specimens color-coded by species
species.colz <- as.character(spec.info$Species.col)
plotTangentSpace(oligocottinae.gpa$coords, groups = species.colz)
#
## END STEP 1.1
#
#### STEP 1.2: Analysis of symmetry
#
# Make a geomorph data frame to hold all the info we want to use
oligocottinae.gdf <- geomorph.data.frame(coords = oligocottinae.gpa$coords,
                    length = spec.info$Length,
                    sex = as.factor(as.character(spec.info$Sex)),
                    isolate.side = as.character(spec.info$Isoalate.side),
                    side = as.character(spec.info$Side),
                    replicate = rep(c(1,2,3),times = nrow(spec.info)/3),
                    isolate = as.factor(as.character(spec.info$Isolate)))
#
#' Now we'll use the "bilat.symmetry" function from geomorph to find the mean postition
#' of each landmark across the three replicates of each preopercle, then the symmetric 
#' position of each landmark across the two sides (i.e., the mean position of left and 
#' reflected right) of each specimen. The "bilat.symmetry" function performs several
#' tests and provides additional data that we will use in later sections as well.
oligocottinae.symm <- bilat.symmetry(A = oligocottinae.gdf$coords,
                   ind = oligocottinae.gdf$isolate,
                   side = oligocottinae.gdf$side,
                   replicate = oligocottinae.gdf$replicate,
                   object.sym = F, RRPP = T, ProcD = F)
summary(oligocottinae.symm)
#
## END STEP 1.2
#
#### END STEP 1 ####
#
######### STEP 2: Analyses of intraspecific variation ############
#' This section corresponds to the "Intraspecific variation" section in the methods section.
#' 
#' Read in species info. These are the same data as before, but re-formatted and subset
#' so that there is only a single entry per specimen.
spec.info.lr.noreps <- read.csv(file = "Supplementary_materials_1.5_specimen_info_symm.csv", header = T, row.names = 1)
#
# Now re-order to match the landmark dataset
spec.info.lr.noreps <- spec.info.lr.noreps[dimnames(oligocottinae.symm$symm.shape)[[3]],]
dimnames(oligocottinae.symm$symm.shape)[[3]] == row.names(spec.info.lr.noreps) # Double-check, looks good!
#
# Make a geomorph data frame to hold all the info we want to use
oligocottinae.symm.gdf <- geomorph.data.frame(coords = oligocottinae.symm$symm.shape,
                                                length = spec.info.lr.noreps$Length,
                                                sex = spec.info.lr.noreps$Sex,
                                              species = spec.info.lr.noreps$Species)
#'
#### STEP 2.1: Allometric growth of the shape of the preopercle
#'
##' STEP 2.1.1: Homogeneity of slopes test
#' We'll first test for evidence that all oligocottines share a common allometric growth trajectory.
#' To do so, we'll test for a relationship between the standard length of a specimen and the shape
#' of its preopercle with the oligocottine species either given a common growth trajectory (i.e.,
#' slope of the regression line) or each allowed a unique growth trajectory. 
oligocottinae.symm.allo.common <- procD.lm(coords ~ log(length) + species, data = oligocottinae.symm.gdf) # common slope across species
oligocottinae.symm.allo.unique <- procD.lm(coords ~ log(length) * species, data = oligocottinae.symm.gdf) # unique slope possibile for each species
#
summary(oligocottinae.symm.allo.common)
summary(oligocottinae.symm.allo.unique) # the interaction between size and species is significant
#'
#' We'll use the "anova" function to test whether adding the interaction term results in a significant 
#' decrease in the sum of squared error. A significant p-value allows us to reject the null hypothesis
#' that all species share a common allometric trajectory. 
anova(oligocottinae.symm.allo.common, oligocottinae.symm.allo.unique)
#' The p-value is significant (p = 0.001, R-sq = 0.035947).
#'
#' Let's visualize the allometric variation using the "shape score" projection method described
#' in Drake, A.G. and Klingenberg, C.P., 2008. The pace of morphological change: historical 
#' transformation of skull shape in St Bernard dogs. Proceedings of the Royal Society B: 
#' Biological Sciences, 275(1630), pp.71-76. This method projects each indivdidual into a space
#' defined by the (log) standard length of the indivudal (x-axis) and a shape variable associated
#' with the predicted shape changes of the regression model and the residual variation thereof 
#' (y-axis). We'll use the "plotAllometry" function in geomorph to do this: 
oligocottinae.symm.allo.unique.reg <- plotAllometry(oligocottinae.symm.allo.unique, size = oligocottinae.symm.gdf$length, 
              logsz = T, method = "RegScore",
              pch = 19, col = as.character(spec.info.lr.noreps$Species.col))
#'
#' This is nice, but let's add regression lines for each species to really help us  visually 
#' assess and compare the allometric trajectories
#' 
#' List of unique species in the dataset
species.total <- as.character(unique(spec.info$Species))
#'
#' A simple loop to polulate the plot with the species-specific trajectories
for(i in 1:length(species.total)){
lm.holder <- lm(oligocottinae.symm.allo.unique.reg$RegScore[spec.info.lr.noreps$Species==species.total[i]]~ 
                  log(spec.info.lr.noreps$Length[spec.info.lr.noreps$Species==species.total[i]]))
lines(y = c(min(lm.holder$fitted.values), max(lm.holder$fitted.values)), 
      x = c(min(lm.holder$model[2]), max(lm.holder$model[2])), 
      col = as.character(unique(spec.info.lr.noreps$Species.col[spec.info.lr.noreps$Species==species.total[i]])),
      lwd = 3
)
}
#' There is clearly a great deal of diversity in allometric trajectories across
#' oligocottine species.
#' 
#' Now, let's visualize the shape variation associated with changes in length. We'll
#' do so using both TPS transformation grids as well as with a 3D model of an average-
#' shaped preopercle bone. 
#' 
#' First, we'll read-in a .PLY formatted 3D model of a preopercle from an individual 
#' with an average-ish shaped preopercle.
Alateralis_TBGroup_050_F <-read.ply(file = "Supplementary_materials_1.6_Alateralis_TBGroup_050_F.ply")
#'
#' Grab the landmark coordinates from one of the replicates of this specimen
Alateralis_TBGroup_050_F.coords <- resample.test1[,,"Alateralis_104831_TBGroup_050_T_rightpreopercle_isolated_1"]
#' 
#' Let's view the preopercle with the landmarks plotted on it
plotspec(Alateralis_TBGroup_050_F, digitspec = resample.test1[,,"Alateralis_104831_TBGroup_050_T_rightpreopercle_isolated_1"]) # Looks nice!
#'
#' Now we'll calculate the mean position of each landmark across all the specimens in our dataset.
#' Together, these mean positions describe the mean preopercle shape for Oligocottinae.
oligocottinae.symm.mean.shape.coords <- mshape(oligocottinae.symm$symm.shape)
#'
#' Now we'll transform the preopercle specimen that we read-in earlier to take on the mean
#' shape of Oligocottinae.
oligocottinae.symm.mean.shape.mesh <- warpRefMesh(Alateralis_TBGroup_050_F, Alateralis_TBGroup_050_F.coords, oligocottinae.symm.mean.shape.coords)
#
#' Now we'll use the "shape.predictor" function to estimate the landmark positions we'd expect to find
#' at very small or very large body sizes, given the allometric model that we've put together.
oligocottinae.symm.allo.unique.reg.pred <-    shape.predictor(oligocottinae.symm.allo.unique.reg$GM$fitted, 
                                               x = oligocottinae.symm.allo.unique.reg$RegScore,
                                               predmin = min(oligocottinae.symm.allo.unique.reg$RegScore),
                                               predmax = max(oligocottinae.symm.allo.unique.reg$RegScore))
#' 
#' Now we can use these predicted landmark positions to visualize the change in shape
#' from the mean preopercle shape to the predicted shape of the preopercle in very 
#' small individuals vs very large individuals according to our model of allometric growth.                 
#'
#' We'll start with a TPS grid transformation of from the mean shape to low values
#' of the regression (i.e., small individuals)
plotRefToTarget(oligocottinae.symm.mean.shape.coords, 
                oligocottinae.symm.allo.unique.reg.pred$predmin,
                mag=1)
title(sub = "Oligocottinae Symm Shape-Allom Min Value")
#'
#' Now the same transformation applied to the 3D model
plotRefToTarget(M1 = oligocottinae.symm.mean.shape.coords, 
                M2 = oligocottinae.symm.allo.unique.reg.pred$predmin,
                mesh = oligocottinae.symm.mean.shape.mesh,
                method = "surface",
                mag=1)
#'
#' Next, we'll look at a TPS grid transformation from the mean shape to high
#' values of the regression (i.e., large individuals)
plotRefToTarget(oligocottinae.symm.mean.shape.coords, 
                oligocottinae.symm.allo.unique.reg.pred$predmax,
                mag=1)
title(sub = "Oligocottinae Symm Shape-Allom Max Value")
#' 
#' Now the same transformation applied to the 3D model
plotRefToTarget(M1 = oligocottinae.symm.mean.shape.coords, 
                M2 = oligocottinae.symm.allo.unique.reg.pred$predmax,
                mesh = oligocottinae.symm.mean.shape.mesh,
                method = "surface",
                mag=1)
#
#
#' END STEP 2.1.1
#'
##' STEP 2.1.2: Species-specific allometric tests
#'
#' We have rejected the hypothesis that size and shape do not covary in oligocottines
#' as well as the hypothesis that oligocottines share a common allometric
#' trajectory, but we have not investigated the potential for variation in allometry 
#' at the species level. To do so, we'll agaian perform a multivariate regression of 
#' preopercle shape using the length of each specimen as the predictor variable, but
#' we'll do so for each species inividually.
#
#' Use a simple for loop to create species-specific subsets landmark coordinates dataset
for(i in 1:length(species.total)){
  working.specimens.loop <- row.names(spec.info[spec.info$Species == species.total[i],])   
  assign(species.total[i], resample.test1[,,working.specimens.loop], envir = .GlobalEnv)
}
#' 
#' Use a simple for loop to create species-specific subsets of the specimen information
#' dataset
for(i in 1:length(species.total)){
  assign(paste(species.total[i],".info", sep = ""), spec.info[dimnames(get(species.total[i]))[3][[1]],], envir = .GlobalEnv)
}
#'
#' Use another simple loop to perform a generalized Procrustes
#' alignment for each of the species-specific landamrk coordinates
#' datasets
for(i in 1:length(species.total)){
  assign(paste(species.total[i],".gpa", sep = ""), gpagen(A=get(species.total[i]), curves = curvesliders, ProcD = F)  ,envir = .GlobalEnv )
}
#
#' Now we'll use a more complex loop to make a geomorph dataframe and perform 
#' the symmetry analysis for each species-specific dataset.
for(i in 1:length(species.total)){
working.coords.holder <- get(paste(species.total[i],".gpa", sep = ""))
working.info.holder <- get(paste(species.total[i],".info", sep = ""))
#'
#' Geomorph data frame
assign(paste(species.total[i],".gdf", sep = ""),
  geomorph.data.frame(coords = working.coords.holder$coords,
                                         length = working.info.holder$Length,
                                         sex = as.factor(as.character(working.info.holder$Sex)),
                                         isolate.side = as.character(working.info.holder$Isoalate.side),
                                         side = as.character(working.info.holder$Side),
                                         replicate = rep(c(1,2,3),times = nrow(working.info.holder)/3),
                                         isolate = as.factor(as.character(working.info.holder$Isolate))))
#
working.gdf <- get(paste(species.total[i],".gdf", sep = ""))
#'
#' Symmetry analysis
assign(paste(species.total[i],".gpa.symm", sep = ""),
    bilat.symmetry(A = working.gdf$coords,
                                     ind = working.gdf$isolate,
                                     side = working.gdf$side,
                                     replicate = working.gdf$replicate,
                                     object.sym = F, RRPP = T, ProcD = F))
}
#
#' Now we'll test for allometric growth of the preopercle within each
#' species in Oligocottinae seperately.
for(i in 1:length(species.total)){
  working.symm.coords.holder <- get(paste(species.total[i],".gpa.symm", sep = ""))$symm.shape
  working.symm.info.holder <- spec.info.lr.noreps[dimnames(working.symm.coords.holder)[[3]],]
# Create a geomorph data frame to hold the pertinent info 
  assign(paste(species.total[i],".gpa.symm.gdf", sep = ""), 
         geomorph.data.frame(coords = working.symm.coords.holder,
                             length = working.symm.info.holder$Length,
                             sex = working.symm.info.holder$Sex
         ),
         envir = .GlobalEnv)
# Perform the regression  
  assign(paste(species.total[i],".gpa.symm.allometry", sep = ""), 
         procD.lm(coords ~ length, data = get(paste(species.total[i],".gpa.symm.gdf", sep = ""))
         ),
         envir = .GlobalEnv)
# Print the species name and summary statistics for each regression  
  print(species.total[i])
  print(summary(get(paste(species.total[i],".gpa.symm.allometry", sep = ""))))
}  
#
#' END STEP 2.1.2
#'
###########################
#'
#' STEP 2.1.3: Visualize allometric shape change with TPS transformation grids
#' 
#' 
currentwd <-  getwd()
dir.create(paste(currentwd, "/Species_Allometry_TPStransforms", sep = ""))
setwd(paste(currentwd, "/Species_Allometry_TPStransforms", sep = ""))
#'
#'
for(i in 1:length(species.total)){
  working.symm.coords.holder <- get(paste(species.total[i],".gpa.symm", sep = ""))$symm.shape
  working.symm.info.holder <- spec.info.lr.noreps[dimnames(working.symm.coords.holder)[[3]],]
# Use fitted values from the model to make prediction lines
pdf(file = paste(species.total[i],".gpa.symm.allometry.plot.pdf", sep = ""), 
    width = 11, height = 8.5, useDingbats = F)
assign(paste(species.total[i],".gpa.symm.allometry.plot", sep = ""),
       plot(get(paste(species.total[i],".gpa.symm.allometry", sep = "")),
            type = "regression", 
            predictor = working.symm.info.holder$Length,
            reg.type = "RegScore", 
            pch = 19, col = "blue"))
abline(lm(get(paste(species.total[i],".gpa.symm.allometry.plot", sep = ""))$RegScore ~
            working.symm.info.holder$Length), col = "red")
title(main= paste(species.total[i], " Symmetric Shape Allometry", sep = ""))
dev.off()

write.csv(get(paste(species.total[i],".gpa.symm.allometry.plot", sep = ""))$RegScore, file = paste(species.total[i],"gpa.symm.allometry.RegScores.csv", sep = ""))

# Make an object to hold the average symm shape
assign(paste(species.total[i],".gpa.symm.M", sep = ""),
       mshape(working.symm.coords.holder),
       envir = .GlobalEnv)

# Calculate shape change for min and max scores in our regression model
assign(paste(species.total[i],".gpa.symm.allometry.preds", sep = ""),
       shape.predictor(get(paste(species.total[i],".gpa.symm.allometry", sep = ""))$GM$fitted, 
                       x = get(paste(species.total[i],".gpa.symm.allometry.plot", sep = ""))$RegScore,
                       predmin = min(get(paste(species.total[i],".gpa.symm.allometry.plot", sep = ""))$RegScore),
                       predmax = max(get(paste(species.total[i],".gpa.symm.allometry.plot", sep = ""))$RegScore)
       ),
       envir = .GlobalEnv)

# Save a pdf of the shape change associated with low values of the regression
pdf(file = paste(species.total[i],"gpa.symm.allometry.minVal.pdf", sep = ""), 
    width = 11, height = 8.5, useDingbats = F)
plotRefToTarget(get(paste(species.total[i],".gpa.symm.M", sep = "")), 
                get(paste(species.total[i],".gpa.symm.allometry.preds", sep = ""))$predmin,
                mag=1)
title(sub = paste(species.total[i], " Allom. Symm. Min Value", sep = ""))
dev.off()

# Save a pdf of the shape change associated with high values of the regression
pdf(file = paste(species.total[i],".gpa.symm.allometry.maxVal.pdf", sep = ""), 
    width = 11, height = 8.5, useDingbats = F)
plotRefToTarget(get(paste(species.total[i],".gpa.symm.M", sep = "")), 
                get(paste(species.total[i],".gpa.symm.allometry.preds", sep = ""))$predmax,
                mag=1)
title(sub = paste(species.total[i], " Allom. Symm. Max Value", sep = ""))
dev.off()
}
#
setwd(currentwd)

#######################################################################
#######################################################################

##### Now let's illustrate shape change using a 3D model

dir.create(paste(currentwd, "/Species_Allometry_PLYtransforms", sep = ""))
setwd(paste(currentwd, "/Species_Allometry_PLYtransforms", sep = ""))


for(i in 1:length(species.total)){
  
  # Warp the species specimen mesh to take on the average shape of the whole species
  assign(paste(species.total[i],".mesh.ave", sep = ""), 
         warpRefMesh(mesh = Alateralis_TBGroup_050_F,
                     mesh.coord =  Alateralis_TBGroup_050_F.coords,
                     ref= get(paste(species.total[i],".gpa.symm.M", sep = ""))),
         envir = .GlobalEnv )
  rgl.close()
  rgl.close()
  
  #Export untransformed surface
  plotRefToTarget(get(paste(species.total[i], ".gpa.symm.M",sep = "")), 
                  get(paste(species.total[i], ".gpa.symm.M",sep = "")),
                  mesh = get(paste(species.total[i], ".mesh.ave",sep = "")), 
                  method="surface")
  writePLY(paste(species.total[i], ".Symm.Allo.ave",  ".mesh.ply", sep = ""), format = "little_endian")
  rgl.close()
  
  # Save a ply model of the shape change associated with low values of the regression
  plotRefToTarget(get(paste(species.total[i], ".gpa.symm.M",sep = "")), 
                  get(paste(species.total[i],".gpa.symm.allometry.preds", sep = ""))$predmin,
                  mesh = get(paste(species.total[i], ".mesh.ave",sep = "")), 
                  method="surface",
                  mag=1)
  writePLY(paste(species.total[i],".Symm.Allo.MinVal.mesh.ply", sep = ""), format = "little_endian")
  rgl.close()
  
  
  # Save a ply model of the shape change associated with high values of the regression
  plotRefToTarget(get(paste(species.total[i], ".gpa.symm.M",sep = "")), 
                  get(paste(species.total[i],".gpa.symm.allometry.preds", sep = ""))$predmax,
                  mesh = get(paste(species.total[i], ".mesh.ave",sep = "")), 
                  method="surface",
                  mag=1)
  writePLY(paste(species.total[i],".Symm.Allo.MaxVal.mesh.ply", sep = ""), format = "little_endian")
  rgl.close()
  
}
setwd(currentwd)
## END STEP 2.1
#' 
#### STEP 2.2: Sexual dimorphism
#' We'll use a loop to test for sexual dimorphism in preopercle shape for each
#' species individually.
for(i in 1:length(species.total)){

  print(i)
  print(species.total[i])
  
  #' We don't want to include juveniles in this analysis, so
  #' we'll make an adults-only dataset for each species 
  working.info.holder <- spec.info.lr.noreps[dimnames(get(paste(species.total[i],".gpa.symm", sep = ""))$symm.shape)[[3]],]
  working.info.holder <- working.info.holder[working.info.holder$Sex != "JUV",] # remove juveniles
  
  working.coords.holder <- get(paste(species.total[i],".gpa.symm", sep = ""))$symm.shape[,,row.names(working.info.holder)]
  
  # Make a dataframe 
  assign(paste(species.total[i],".gdf", sep = ""), 
         geomorph.data.frame(shape = working.coords.holder,
                             fish.length = working.info.holder$Length,
                             fish.sex =    as.factor(as.character(working.info.holder$Sex))
         ),
         envir = .GlobalEnv)
  
  
  # Test for sexual dimorphism
  assign(paste(species.total[i],".sex.dimorph", sep = ""), 
         procD.lm(shape~fish.sex, 
                  data = get(paste(species.total[i], ".gdf", sep = "")),
                  iter=999, print.progress = FALSE),
         envir = .GlobalEnv)
  print(paste(species.total[i],".sex.dimorph", sep = ""))  
  print(summary(get(paste(species.total[i],".sex.dimorph", sep = ""))) ) 
  
  # Sexual dimorphism- disparity analysis
  assign(paste(species.total[i],".sex.dimorph.disp", sep = ""), 
         morphol.disparity(shape~1, groups = get(paste(species.total[i], ".gdf", sep = ""))$fish.sex, 
                           data = get(paste(species.total[i], ".gdf", sep = "")),
                           iter=999, print.progress = FALSE),
         envir = .GlobalEnv)
  print(paste(species.total[i],".sex.dimorph.disp", sep = ""))  
  print(summary(get(paste(species.total[i],".sex.dimorph.disp", sep = ""))) ) 
  
  #' Check the full model-- see whether males and females have
  #' different allometric trajectories
  assign(paste(species.total[i],".allometry.full", sep = ""), 
         procD.lm(shape~fish.length*fish.sex, 
                  data = get(paste(species.total[i], ".gdf", sep = "")),
                  iter=999, print.progress = FALSE),
         envir = .GlobalEnv)
  print(paste(species.total[i],".allometry.full", sep = ""))  
  print(summary(get(paste(species.total[i],".allometry.full", sep = ""))))  
  
  #' Check the reduced model-- see whether the adults-only dataset changed
  #' the outcome of the test for allometry
  assign(paste(species.total[i],".allometry.reduced", sep = ""), 
         procD.lm(shape~fish.length, 
                  data = get(paste(species.total[i], ".gdf", sep = "")),
                  iter=999, print.progress = FALSE),
         envir = .GlobalEnv)
  print(paste(species.total[i],".allometry.reduced", sep = ""))  
  print(summary(get(paste(species.total[i],".allometry.reduced", sep = ""))) ) 
  
}  


#### STEP 2.3: Fluctuating asymmetry
#' For each species, we'll first calculate the Procrustes distance between the
#' asymmetric component of the shape of the preopercle and the mean directional
#' asymmetric shape of its species. We'll then divide this number by the avearge
#' Procrustes distance from the mean shape for that species to give us the 
#' relative fluctuating asymmetry (rFA) for that individual. We'll then calculate
#' the average rFA for each species.
#' 
#' Set up a table to summarize the results of the "distance from the mean shape"
#' for each species.
d.summary.table <- matrix(data = NA, nrow = 17, ncol = 8)
row.names(d.summary.table) <- species.total
colnames(d.summary.table) <- c("DistFmean.R.min", "DistFmean.R.max", "DistFmean.R.median", "DistFmean.R.mean",
                               "DistFmean.L.min", "DistFmean.L.max", "DistFmean.L.median", "DistFmean.L.mean")

#' Use a loop to calculate the disantance from the mean shape for
#' each species and populate the table
for(i in 1:length(species.total)){
  
  species.info.holder <- get(paste(species.total[i],".info", sep = ""))
  species.coords.holder <- get(paste(species.total[i],".gpa", sep = ""))$coords
  
  species.coords.holder.R <- species.coords.holder[,,species.info.holder$Side == "Right"]
  species.coords.holder.L <- species.coords.holder[,,species.info.holder$Side == "Left"]
  
  species.dist.fromave.holder.R <- proc.dist.from.ave(species.coords.holder.R)
  assign(paste(species.total[i],".dist.from.ave.R", sep = ""), 
         species.dist.fromave.holder.R,
         envir = .GlobalEnv)
  
  d.summary.table[i,1] <- min(species.dist.fromave.holder.R)
  d.summary.table[i,2] <- max(species.dist.fromave.holder.R)
  d.summary.table[i,3] <- median(species.dist.fromave.holder.R)
  d.summary.table[i,4] <- mean(species.dist.fromave.holder.R)
  
  species.dist.fromave.holder.L <- proc.dist.from.ave(species.coords.holder.L)
  assign(paste(species.total[i],".dist.from.ave.L", sep = ""), 
         species.dist.fromave.holder.L,
         envir = .GlobalEnv)
  
  d.summary.table[i,5] <- min(species.dist.fromave.holder.L)
  d.summary.table[i,6] <- max(species.dist.fromave.holder.L)
  d.summary.table[i,7] <- median(species.dist.fromave.holder.L)
  d.summary.table[i,8] <- mean(species.dist.fromave.holder.L)
  
}

#' Now we'll make a simple function to calculate the fluctuating asymmetry
#' of each individual of each species.
procD.FA <- function(asym.data.array, DA.data.array){
  
  ave.DA <- mshape(DA.data.array)
  name.dummy <- dimnames(asym.data.array)
  name.vector <- name.dummy[[3]]
  comp.matrix <- matrix(data = 0, nrow = length(name.dummy[[3]]), ncol = 1)
  colnames(comp.matrix) <- c("ProcD.Asym.to.aveDA")
  row.names(comp.matrix) <- name.dummy[[3]]
  
  
  for (j in 1:length(name.vector)){
    
    comp.matrix[j,] <- procdist(ave.DA, asym.data.array[,,name.vector[[j]]])
    
  }
  
  return(comp.matrix)
}

#' We'll set up a matrix to hold the summary data for each species
Pdist.FA.table <- matrix(data = NA, nrow = 17, ncol = 3)
row.names(Pdist.FA.table) <- species.total
colnames(Pdist.FA.table) <- c("Mean.ProcD.FA", "Max.Mean.ProcD.Side", "Mean.Rel.ProcD.FA")

#' Now we'll use a loop to apply our function to each species and store
#' the species-specific results for each species as new objects as well as
#' save the summary results in the "Pdist.FA.table" matrix that we just set up.
for(i in 1:length(species.total)){
  working.coords.holder <- get(paste(species.total[i],".gpa.symm",sep = ""))
  assign(paste(species.total[i], ".gpa.symm.procD.FA", sep = ""),
    procD.FA(asym.data.array= working.coords.holder$asymm.shape, DA.data.array= working.coords.holder$DA.component))
   
  Pdist.FA.table[i,1] <- mean(get(paste(species.total[i], ".gpa.symm.procD.FA", sep = "")))
  Pdist.FA.table[i,2] <- max(d.summary.table[i,4], d.summary.table[i,8])
  Pdist.FA.table[i,3] <- mean( (get(paste(species.total[i], ".gpa.symm.procD.FA", sep = "")))/ Pdist.FA.table[i,2] )
  
  #' Here, we'll create a table of the fluctuating asymmetry (ProcD.Asym.to.aveDA) and 
  #' relativized fluctuating asymmetry (Rel.ProcD.FA) of each specimen for a given 
  #' species, and save that table as a new object.    
  assign(
      paste(species.total[i], ".gpa.symm.procD.FA", sep = ""),
      cbind(get(paste(species.total[i], ".gpa.symm.procD.FA", sep = "")),"Rel.ProcD.FA" = (get(paste(species.total[i], ".gpa.symm.procD.FA", sep = ""))[,1])/ Pdist.FA.table[i,2] )
      )
  
}

########################'

#' We can also take a look at the rFA for each individual of each species all 
#' together.
start.matrix <- get(paste(species.total[1], ".gpa.symm.procD.FA", sep = ""))
for(i in 2:length(species.total)){
  start.matrix1 <- get(paste(species.total[i], ".gpa.symm.procD.FA", sep = ""))
  start.matrix <- rbind(start.matrix, start.matrix1)
  }
start.matrix # take a look
#
## END STEP 2.3
#
#### END STEP 2 ####
#
######### STEP 3: Phylogenetic comparative methods ############
#' This section corresponds to the "Macroevolutionary hypotheses" section in the methods section.
#' 
#' We want to compare species here, so the first thing we'll do is find the mean preopercle
#' shape for each species.
#' 
## Set up an array to hold the landmark coordinates for the average shape of each species
Oligocottinae.symm.means <- array (dim = c(89, 3, 17), dimnames = list(1:89, c("x", "y","z"), species.total))
#
## Now we'll populate it with a loop. We'll take the mean shape of the symmetric component
#' of the preopercle shape for each species.
for (i in 1:17) {
  Oligocottinae.symm.means [,,i] <-mshape(get(paste(species.total[i],".gpa.symm", sep = ""))$symm.shape)
}
#'
#' Now we'll perform a Procrustes superimposition of the landmark coordinates
Oligocottinae.symm.means.gpa <- gpagen(Oligocottinae.symm.means, curves = curvesliders, ProcD = F)
#'
#' For ease of visualization, we'll make an object to color-code our species
#' for plotting
species.colz <- as.character(unique(spec.info.lr.noreps$Species.col))
#'
#' Let's perform a principal component analysis and visualize the morphospace
#' of the first two PC axes
Oligocottinae.means.gpa.pca <- plotTangentSpace(Oligocottinae.symm.means.gpa$coords,
                                                groups = species.colz,
                                                label = names(Oligocottinae.symm.means.gpa$Csize))
title(main = "Oligocottinae Symm Mean")
#'
#' Read in MCC phylogeny from "Buser et al. (2017), Littorally adaptive? 
#' Testing the link between habitat, morphology, and reproduction in the 
#' intertidal sculpin subfamily Oligocottinae (Pisces: Cottoidea). 
#' PeerJ 5:e3634; DOI 10.7717/peerj.3634." This phylogeny contains all species 
#' of the subfamily Oligocottinae as well as several outgroup taxa. 
sculpin.tree <- read.nexus(file = "Supplementary_materials_1.7_birthdeath_mcc_starbeast.tree")
#'
#' For the purposes of analysis, we will extract the clade containing Oligocottinae 
#' and the outgroup taxon Chitonotus pugetensis
Oligo_tree <- extract.clade(sculpin.tree, 52)
plotTree(Oligo_tree) # Take a look at the tree
#'
#' Read in the trait data for each species
Oligocottinae.species.data <- read.csv(file = "Supplementary_materials_1.8_Oligocottinae.info.matrix.csv", header = T, row.names = 1)

#' Extract the coordinate data and then rearrange it to match 
#' the species order of the phylogeny
Oligocottinae.symm.means.gpa.coords <- Oligocottinae.symm.means.gpa$coords[,,c(1:5,10,6:9,11:17)]
dimnames(Oligocottinae.symm.means.gpa.coords)[[3]] <- Oligo_tree$tip.label #rename
row.names(Oligocottinae.species.data) == Oligo_tree$tip.label #all in the same order
#'
#' Create a data frame to hold all the data
Oligo.symm.means.gdf <- geomorph.data.frame(coords = Oligocottinae.symm.means.gpa.coords,
                                            shape.allo = Oligocottinae.species.data$shape.allometry,
                                            max.size = Oligocottinae.species.data$max.size.mm,
                                            sex.index = Oligocottinae.species.data$s.selec.index,
                                            mean.rFA = Oligocottinae.species.data$Mean.rFA
)
#'
#' Now we can plot the phylomorphospace using the same color coding as before
dev.off()
plotGMPhyloMorphoSpace(Oligo_tree, Oligocottinae.symm.means.gpa.coords, 
                       tip.labels = T, node.labels = F,
                       plot.param = list(t.bg=as.character(Oligocottinae.species.data$Species.col),   #to change overlay, just change this list to a different variable color
                                         txt.cex=0.5)
)
title(main = "Symmetric Shape, Species Average- Species Coloration")                       
#'
#' Now let's show the shape change associated with the first few PC axes
# Find the mean specimen
findMeanSpec(Oligocottinae.symm.means.gpa.coords)
#Clinocottus_analis
# Read in the ply of an average specimen of C. analis and plot the landmarks on it
Canalis_TBG060T_Right <- read.ply(file= "Supplementary_materials_1.9_Canalis_TBG060T_Right.ply") # read in specimen
plotspec(Canalis_TBG060T_Right, resample.test1[,,"Canalis_1_105007_TBGroup_060_T_rightpreopercle_isolated_3"]) # plot landmarks
#'
#' Calculate the average shape for the preopercle in Oligocottinae
Oligocottinae.means.gpa.mshape <-mshape(Oligocottinae.symm.means.gpa.coords)
#'
#' Warp the preopercle from the C. analis specimen to take on the average shape
Oligocottinae.means.gpa.mshape.mesh <- warpRefMesh(mesh = Canalis_TBG060T_Right, 
                                                   mesh.coord = resample.test1[,,"Canalis_1_105007_TBGroup_060_T_rightpreopercle_isolated_3"],
                                                   Oligocottinae.means.gpa.mshape)
#' Now let's visualize an untransformed TPS grid and the average-shaped preopercle.
#' We'll tranform both the TPS grid and the average preopercle to illustrate the
#' changes in shape associated with the first few PC axes.
# Untransformed grid
plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.mshape, Oligocottinae.means.gpa.mshape.mesh, method = "TPS")
# Untranformed POP
Oligocottinae.means.gpa.mshape.mesh <- warpRefMesh(mesh = Canalis_TBG060T_Right, 
                                                   mesh.coord = resample.test1[,,"Canalis_1_105007_TBGroup_060_T_rightpreopercle_isolated_3"],
                                                   Oligocottinae.means.gpa.mshape)
#
# PC1 min
plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.pca$pc.shapes$PC1min, Oligocottinae.means.gpa.mshape.mesh, method = "TPS")
Oligocottinae.means.gpa.pca.PC1min.mesh <- plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.pca$pc.shapes$PC1min, Oligocottinae.means.gpa.mshape.mesh, method = "surface")
# PC1 max
plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.pca$pc.shapes$PC1max, Oligocottinae.means.gpa.mshape.mesh, method = "TPS")
Oligocottinae.means.gpa.pca.PC1max.mesh <- plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.pca$pc.shapes$PC1max, Oligocottinae.means.gpa.mshape.mesh, method = "surface")
# PC2 min
plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.pca$pc.shapes$PC2min, Oligocottinae.means.gpa.mshape.mesh, method = "TPS")
Oligocottinae.means.gpa.pca.PC2min.mesh <- plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.pca$pc.shapes$PC2min, Oligocottinae.means.gpa.mshape.mesh, method = "surface")
# PC2 max
plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.pca$pc.shapes$PC2max, Oligocottinae.means.gpa.mshape.mesh, method = "TPS")
Oligocottinae.means.gpa.pca.PC2max.mesh <- plotRefToTarget(Oligocottinae.means.gpa.mshape , Oligocottinae.means.gpa.pca$pc.shapes$PC2max, Oligocottinae.means.gpa.mshape.mesh, method = "surface")

#################

summary(procD.pgls(coords ~ max.size + sex.index + mean.rFA, phy = Oligo_tree, data = Oligo.symm.means.gdf, iter = 10000))
summary(procD.pgls(coords ~ max.size, phy = Oligo_tree, data = Oligo.symm.means.gdf, iter = 10000))
summary(procD.pgls(coords ~ sex.index, phy = Oligo_tree, data = Oligo.symm.means.gdf))
summary(procD.pgls(coords ~ mean.rFA, phy = Oligo_tree, data = Oligo.symm.means.gdf))

plotGMPhyloMorphoSpace(Oligo_tree, Oligocottinae.symm.means.gpa.coords, 
                       tip.labels = T, node.labels = F,
                       plot.param = list(t.bg=as.character(Oligocottinae.species.data$Mean.rFA.col),   #to change overlay, just change this list to a different variable color
                                         txt.cex=0.5)
)
title(main = "Symmetric Shape, Species Average- Average rFA Coloration")                       


